package mastermind.logic;

import java.io.File;
import java.io.Serializable;
import mastermind.model.Settings;

/**
 * Mastermind Settings: Handler für die Optionen
 */
/* -----------------------------------------------------------------------------
 * Version History:
 * 2012-05-03.v01   FeRo    init version
 * 2012-05-05.v01   FeRo    implement Singleton
 * 2012-05-18.v01   FeRo    Serializable
 * 2012-05-24.v01   FeRo    Save / Load Settings
 * -----------------------------------------------------------------------------
 * @author Felix Rohrer <felix.rohrer@stud.hslu.ch>
 * @version 2012-05-24.v01
 */
public class SettingsHandler implements Serializable
{

    private static final String SAVE_FILENAME = "config.dat";
    private static SettingsHandler instance = null;
    private Settings settings;

    /**
     * Default Konstruktor, kann nicht ausserhalb der Klasse aufgerufen werden.
     */
    public SettingsHandler()
    {
        settings = new Settings();
    }

    /**
     * Statische Methode, liefert die einzige Instanz von Settings zurück
     * @return Single-Instanz von Settings
     */
    public static SettingsHandler getInstance()
    {
        if (instance == null) {
            instance = new SettingsHandler();
        }
        return instance;
    }

    /**
     * Liefert ein Settings Objekt zurück
     * @return Settings-Objekt
     */
    public Settings getSettings()
    {
        return settings;
    }

    /**
     * Ueberprueft die Parameter auf ihre gueltigkeit, Plausibilitaetspruefung.
     * @return true falls die Parameter gültig sind, sonst false
     */
    public boolean validSettings()
    {
        // prevent infinite loop | i.e. genRandomGuess()
        if (( ! settings.getAllowDuplicateColor()) && (settings.getNumberOfColors() < settings.getNumberOfPegs())) {
            return false; // false - Settings Config ist ungültig
        }
        return true; // true - Settings Config ist gültig
    }

    /**
     * Speichert die aktuelle Settings in eine Datei.
     */
    public void saveSettings()
    {
        System.out.println("Save Settings...");
        File file = new File(SAVE_FILENAME);
        //System.out.println(file.getAbsolutePath());
        Serializer.Serialize(file, this);
    }

    /**
     * Lädt die Settings aus einer Datei
     */
    public void loadSettings()
    {
        System.out.println("Load Settings...");
        File file = new File(SAVE_FILENAME);
        //System.out.println(file.getAbsolutePath());
        if (file.exists()) {
            try {
                SettingsHandler loadedSettings = (SettingsHandler) Serializer.DeSerialize(file);

                Settings mySettings = loadedSettings.getSettings();
                SettingsHandler.getInstance().getSettings().setDesign(mySettings.getDesign());
                SettingsHandler.getInstance().getSettings().setNumberOfColors(mySettings.getNumberOfColors());
                SettingsHandler.getInstance().getSettings().setAllowDuplicateColor(mySettings.getAllowDuplicateColor());
                SettingsHandler.getInstance().getSettings().setNumberOfPegs(mySettings.getNumberOfPegs());
                SettingsHandler.getInstance().getSettings().setNumberOfRows(mySettings.getNumberOfRows());

            } catch (Exception e) {
                // Error loading Enginge
                System.out.println("Error Loading Settigns!");
            }
        } else {
            System.out.println("File not found: " + file);
            System.out.println("Use Default Settings...");
        }
    }
}
